// ----
// draggablenavbar jquery plugin
// ----
(function ($) {
    $.fn.draggablenavbar = function (options) {
        var opts = $.extend({
            padding_width: 10000,
            initial_left: -10000
        }, options);

        if (typeof $.fn.draggable !== 'function')
            throw 'This plugin depends on jQuery UI Draggable plugin. Please ensure it is referenced first.';

        return this.each(function () {
            var $this = $(this);
            var tagName = $this.get(0).tagName;
            if (tagName !== "OL" && tagName !== "UL") {
                throw new Error("This plugin must be applied to either OL or UL dom elements.");
            }

            // Add padding to beginning and end of nav bar
            $('<li/>', { 'class': 'dragnav-prepadding dragnav-padding' })
                .css('width', opts.padding_width)
                .prependTo($this);
            $('<li/>', { 'class': 'dragnav-postpadding dragnav-padding' })
                .css('width', opts.padding_width)
                .appendTo($this);

            // Add padding placeholder in case of additional styling along gutters
            $this.children('li.dragnav-padding').append($('<div class="dragnav-padding-placeholder"></div>'));

            // Set initial left to skip padding area
            $this.css('left', opts.initial_left);

            $this.draggable({
                axis: 'x',
                start: function (e, ui) {
                    $this.data("mouseEvents", [e]);
                    $this.addClass('navigation_bar-dragging');

                    $(document)
	                   .mousemove(function (e) {
	                       var mouseEvents = $this.data("mouseEvents");
	                       if (e.timeStamp - mouseEvents[mouseEvents.length - 1].timeStamp > 40) {
	                           mouseEvents.push(e);
	                           if (mouseEvents.length > 2) {
	                               mouseEvents.shift();
	                           }
	                       }
	                   })
	                   .mouseup(function () {
	                       $(document).unbind("mousemove mouseup");
	                   });
                },
                stop: function (e, ui) {
                    $this.stop();
                    $this.css("text-indent", 100);
                    var lastE = $this.data("mouseEvents").shift();

                    var x1 = lastE.pageX;
                    var y1 = lastE.pageY;
                    var t1 = lastE.timeStamp;
                    var x2 = e.pageX;
                    var y2 = e.pageY;
                    var t2 = e.timeStamp;
                    var minDistance = 40; // Minimum px distance object must be dragged to enable momentum.

                    // Deltas
                    var dX = x2 - x1,
                        dY = y2 - y1,
                        dMs = Math.max(t2 - t1, 1);

                    // Speeds
                    var speedX = Math.max(Math.min(dX / dMs, 1), -1),
                        speedY = Math.max(Math.min(dY / dMs, 1), -1);

                    // Distance moved (Euclidean distance)
                    var distance = Math.sqrt(Math.pow(x1 - x2, 2) + Math.pow(y1 - y2, 2));

                    if (distance > minDistance) {
                        // Momentum
                        var lastStepTime = new Date();
                        $this.animate({
                            textIndent: 0
                        }, {
                            duration: Math.max(Math.abs(speedX), Math.abs(speedY)) * 3000,
                            easing: 'swing',
                            complete: function () {
                                $this.removeClass('navigation_bar-dragging');
                            },
                            step: function (currentStep) {
                                speedX *= (currentStep / 105);

                                var now = new Date();
                                var stepDuration = now.getTime() - lastStepTime.getTime();

                                lastStepTime = now;

                                var position = $this.position();
                                var newLeft = (position.left + speedX * stepDuration);

                                $this.css({
                                    left: newLeft + "px"
                                });
                            }
                        });
                    } else {
                        window.setTimeout(function () {
                            $this.removeClass('navigation_bar-dragging');
                        }, 125);
                    }
                }
            });

            if (typeof $.fn.addTouch === 'function')
                $this.addTouch();
        });
    };
})(jQuery)